//-----------------------------------------------------------------------
// Canthology: software that makes it easy to create anthologies.
//
// Copyright (C) 2011 Ciaran McHale.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//----------------------------------------------------------------------

package org.canthology.canthology;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.config4j.Configuration;
import org.config4j.ConfigurationException;
import org.config4j.SchemaValidator;


public class Canthology
{
	private static String			cmd;
	private static List				docScopes;
	private static String			cfgFileName;
	private static Configuration	cfg;
	private static int				diagnosticsLevel;
	private static Map				presetVariables;


	public static void main(String[] args)
	{
		int					sIndex;
		int					len;
		String				scope;
		Iterator			iter;
		Map.Entry			entry;
		DocumentInfo		documentInfo;

		parseCmdLineArgs(args);

		if (cmd.equals("-create")) {
			createConfigFile();
			return;
		}

		cfg = Configuration.create();
		try {
			//--------
			// Set all the configration variables specified via
			// "-set <name> <value>" command-line options.
			//--------
			iter = presetVariables.entrySet().iterator();
			while (iter.hasNext()) {
				entry = (Map.Entry)iter.next();
				cfg.insertString("", (String)entry.getKey(),
								 (String)entry.getValue());
			}

			//--------
			// Parse the config file.
			//--------
			cfg.parse(cfgFileName);

			if (docScopes.size() == 0) {
				findAllDocumentScopes();
			}
			if (docScopes.size() == 0) {
				System.err.println("canthology: no document scopes to process");
				System.exit(1);
			}

			//--------
			// Process all the document scopes
			//--------
			len = docScopes.size();
			for (sIndex = 0; sIndex < len; sIndex++) {
				scope = (String)docScopes.get(sIndex);
				if (diagnosticsLevel >= 1) {
					if (sIndex != 0) {
						System.out.println("\n");
					}
					System.out.println("Processing scope '" + scope + "'...");
				}
				documentInfo = new DocumentInfo(cfg, scope, diagnosticsLevel);
				if (!cmd.equals("-validate")) {
					documentInfo.process();
				}
			}
		} catch(CanthologyException ex) {
			System.err.println("canthology: " + ex.getMessage());
			System.exit(1);
		} catch(ConfigurationException ex) {
			System.err.println("canthology: " + ex.getMessage());
			System.exit(1);
		}
	}


	private static void createConfigFile()
	{
		FileWriter		out;
		String			contents;

		contents = StartingPointConfig.getString();
		//--------
		// The call to Util.replace() is required to work around
		// a bug/mis-feature in the current version of the "config2j"
		// utility program provided with "Config4*".
		//--------
		contents = Util.replace(contents, "\n",
			                        System.getProperty("line.separator"));

		try {
			if (diagnosticsLevel >= 1) {
				System.out.println("Creating '" + cfgFileName + "'...");
			}
			out = new FileWriter(cfgFileName);
			out.write(contents);
			out.close();
		} catch(IOException ex) {
			System.err.println("canthology: error writing to file " + ex);
			System.exit(1);
		}
	}


	private static void parseCmdLineArgs(String[] args)
	{
		int					i;
		StringBuffer		msg;

		cmd = "";
		docScopes = new ArrayList();
		cfgFileName = null;
		diagnosticsLevel = 1;
		presetVariables = new HashMap();

		for (i = 0; i < args.length; i++) {
			if (args[i].equals("-h")) {
				usage("");
			} else if (args[i].equals("-d")) {
				if (i == args.length-1) { usage(""); }
				try {
					diagnosticsLevel = Integer.parseInt(args[i+1]);
					if (diagnosticsLevel < 0 || diagnosticsLevel > 3) {
						usage("");
					}
				} catch(NumberFormatException ex) {
					usage("");
				}
				i++;
			} else if (args[i].equals("-create")) {
				if (i == args.length-1) { usage(""); }
				cmd = args[i];
				cfgFileName = args[i+1];
				i++;
			} else if (args[i].equals("-f")) {
				if (i == args.length-1) { usage(""); }
				cfgFileName = args[i+1];
				i++;
			} else if (args[i].equals("-validate")) {
				cmd = args[i];
			} else if (args[i].equals("-scope")) {
				if (i == args.length-1) { usage(""); }
				docScopes.add(args[i+1]);
				i++;
			} else if (args[i].equals("-set")) {
				if (i >= args.length-2) { usage(""); }
				presetVariables.put(args[i+1], args[i+2]);
				i += 2;
			} else {
				usage(args[i]);
			}
		}
		if (cfgFileName == null) {
			if (new File("Canthology.cfg").canRead()) {
				cfgFileName = "Canthology.cfg";
			} else {
				System.err.println("canthology: cannot find 'Canthology.cfg'");
				System.exit(1);
			}
		}
	}


	private static void usage(String optMsg)
	{
		StringBuffer		msg;

		msg = new StringBuffer();
		if (!optMsg.equals("")) {
			msg.append(optMsg + "\n\n");
		}

		msg.append(
		      "usage: canthology [<options>]\n"
		    + "The <options> can be:\n"
		    + "  -h                  Print this usage statement\n"
			+ "  -create <file>      Create a starting-point configuration "
					+ "file\n"
		    + "  -f <file.cfg>       Specify a configuration file ("
					+ "default is 'canthology.cfg')\n"
		    + "  -scope <scope>      Specify a document's scope\n"
		    + "                      (You can specify this multiple "
					+ "times)\n"
		    + "  -d <0..3>           Specify a diagnostics level\n"
		    + "  -validate           Just validate the contents of "
					+ "<file>\n"
		    + "  -set <name> <value> Set a configuration variable\n"
		    + "\n"
		);
		System.err.println(msg.toString());
		System.exit(1);
	}


	private static void findAllDocumentScopes()
	{
		String []			topLevelScopes;
		int					i;

		topLevelScopes = cfg.listFullyScopedNames("", "",
								Configuration.CFG_SCOPE, false);
		for (i = 0; i < topLevelScopes.length; i++) {
			if (cfg.type(topLevelScopes[i], "root_file.base_name")
			    != Configuration.CFG_NO_VALUE)
			{
				docScopes.add(topLevelScopes[i]);
			}
		}
	}

}
